@@ -1,6 +1,7 @@
 import base64
 import json
 from collections import defaultdict
+from itertools import chain
 from typing import Any, List, Optional
 from urllib.parse import urlparse
 
@@ -10,6 +11,8 @@ import requests
 from decouple import config
 from rest_framework.exceptions import ValidationError
 
+from translate.core.utils.logging import log
+
 
 class Client:
     """TranslateClient utils class."""
@@ -52,12 +55,8 @@ class Client:
             host=cache_address, port=cache_port, decode_responses=True
         )
 
-    def __expire(self, *hashes: List[str]):
-        """Expire given hash maps in cache."""
-        for _hash in hashes:
-            self.cache.expire(_hash, Client.CACHE_GROUP_KEY_TTL)
-
-    def __stringify(self, collection: List[Any]) -> List[str]:
+    @staticmethod
+    def __stringify(collection: List[Any]) -> List[str]:
         """Ensure items of a collection are stringified."""
         return [str(item) for item in collection]
 
@@ -67,26 +66,8 @@ class Client:
             bytes(":".join(self.__stringify(hashes)), "utf-8")
         ).decode("utf-8")
 
-    def __set_view(self, language: str, view: str, ids: List[str]) -> Optional[bool]:
-        """Set view with specified TTL."""
-        return self.cache.set(
-            f"{self.__hash(language, view)}",
-            ",".join(self.__stringify(ids)),
-            ex=Client.CACHE_VIEW_TTL,
-        )
-
-    def __set_occurrence(
-        self, language: str, occurence: str, ids: List[str]
-    ) -> Optional[bool]:
-        """Set occurrence with specified TTL."""
-        return self.cache.set(
-            f"{self.__hash(language, occurence)}",
-            ",".join(self.__stringify(ids)),
-            ex=Client.CACHE_OCCURRENCE_TTL,
-        )
-
+    @staticmethod
     def __build_params(
-        self,
         views: List[str],
         occurrences: List[str],
         snake_keys: List[str],
@@ -110,6 +91,29 @@ class Client:
 
         return params
 
+    def __expire(self, *hashes: List[str]):
+        """Expire given hash maps in cache."""
+        for _hash in hashes:
+            self.cache.expire(_hash, Client.CACHE_GROUP_KEY_TTL)
+
+    def __set_view(self, language: str, view: str, ids: List[str]) -> Optional[bool]:
+        """Set view with specified TTL."""
+        return self.cache.set(
+            f"{self.__hash(language, view)}",
+            ",".join(self.__stringify(ids)),
+            ex=Client.CACHE_VIEW_TTL,
+        )
+
+    def __set_occurrence(
+        self, language: str, occurence: str, ids: List[str]
+    ) -> Optional[bool]:
+        """Set occurrence with specified TTL."""
+        return self.cache.set(
+            f"{self.__hash(language, occurence)}",
+            ",".join(self.__stringify(ids)),
+            ex=Client.CACHE_OCCURRENCE_TTL,
+        )
+
     def __load_keys(self, keys: List[str]) -> dict:
         """Load keys from cache."""
         if isinstance(keys, str):
@@ -193,6 +197,52 @@ class Client:
 
         return record
 
+    def _get_all_pages(
+        self,
+        language: str,
+        views: Optional[List[str]] = None,
+        occurrences: Optional[List[str]] = None,
+        snake_keys: Optional[List[str]] = None,
+        page_size: int = 500,
+        page: int = 1,
+        chain_together: bool = True,
+    ) -> Optional[list]:
+        """
+        This method paginates response and returns list of lists of pages
+        """
+        url: str = self._filter_url(language)
+        params = self.__build_params(views, occurrences, snake_keys, page_size, page)
+
+        response = requests.get(url, params=params, timeout=Client.timeout).json()
+        if not response.get("results"):
+            return None
+
+        results = [
+            response,
+        ]
+        count = response.get("count")
+        next_page = response.get("links", {}).get("next")
+
+        while next_page:
+            response = requests.get(next_page, timeout=Client.timeout).json()
+            results.append(response)
+
+            page_num = response.get("pages").get("current")
+            next_page = response.get("links", {}).get("next")
+            log.info(f"Page {page_num} for {language} language.")
+
+        log.info(
+            f"There are no more pages. Total count is {count} translations \
+            with {params} parameters for {language} language."
+        )
+
+        if chain_together:
+            return list(
+                chain.from_iterable([result.get("results") for result in results])
+            )
+
+        return results
+
     def filter(
         self,
         language: str,
@@ -201,8 +251,8 @@ class Client:
         snake_keys: Optional[List[str]] = None,
         keys_number: int = 500,
         no_cache: bool = False,
-        page_size: int = None,
-        page: int = None,
+        page_size: int = 500,
+        page: int = 1,
     ):
         """Filters translations by language, views and occurences."""
         if not language:
@@ -210,9 +260,10 @@ class Client:
 
         if (
             views is None and occurrences is None and snake_keys is None
-        ):  # NOTE: If no filtering parameters given, query service directely.
+        ):  # NOTE: If no filtering parameters given, query service directly.
             no_cache = True
 
+        # NOTE: This part potentially still needs auto-pagination for the query
         if no_cache:
             return requests.get(
                 self._filter_url(language),
@@ -297,14 +348,14 @@ class Client:
                     self.cache.hset(_ID_NAME_MAP, _id_name, _id)
 
                 for view in filter(None, _views):
-                    # self.__set_view(language, view)
                     views[view].append(_id)
                     self.cache.hset(_VIEWS_ID_MAP, _id, view)
+                    self.__expire(_VIEWS_ID_MAP)
 
                 for occurrence in filter(None, _occurrences):
-                    # self.__set_occurrence(language, occur)
                     occurrences[occurrence].append(_id)
                     self.cache.hset(_OCCURRENCES_ID_MAP, _id, occurrence)
+                    self.__expire(_OCCURRENCES_ID_MAP)
 
             # NOTE: Cache freshly retrieved items.
             for view, keys in views.items():
@@ -318,18 +369,12 @@ class Client:
         result = _cache_warmup()
 
         # 5. Fetch the remaining items.
-
-        translations = (
-            requests.get(
-                self._filter_url(language),
-                params=self.__build_params(
-                    views, occurrences, snake_keys, page_size, page
-                ),
-                timeout=Client.timeout,
-            )
-            .json()
-            .get("results")
+        translations = self._get_all_pages(
+            language, views, occurrences, snake_keys, page_size, page
         )
+        if not translations:
+            breakpoint()
+            print
 
         # 6. Cache new translations and assemble output.
         result = {**result, **_cache_translations(translations)}
